def to_lower_case(text_input: str) -> str:
    """Converting text to lowercase

    :param text_input: Input text to be processed.
    :type text_input: `str`
    :return: Text with all characters on lowercase.
    :rtype: `str`
    """
    return text_input.lower()



import typing as tp


def validate_type(line_input: tp.Any, line_id: int = 0) -> None:
    """ Verifies if ´line_input´ is of string or list type.
    
    :param line_input: Line to have its type checked.
    :type line_input: `any`
    :param line_id: Optional line identification.
    :type line_id: `int`
    
    :raise Exception: if the input line is not of a valid listed type.
    """
    valid_types = (str, list)
    input_type = type(line_input)
    
    if input_type not in valid_types:
        raise TypeError(f'Expected input of type string or list, but received a type {input_type} on line {line_id}')



import re


def add_space_to_punctuation(text_input: str) -> str:
    """ Adds a blank space around punctuation and hyphens on input text.
    
    :param text_input: Text to be processed.
    :type text_input: `str`
    
    :return: Processed text with added black spaces before punctuation.
    :rtype: `str`
    """
    processed_punct = add_space_around_punctuation(text_input)
    processed_hyphen = add_space_to_hyphens(processed_punct)
    
    return processed_hyphen.strip()


def add_space_to_hyphens(text_input: str) -> str:
    """ Adds a blank space around hyphens when the character before it is a number.

    :param text_input: Text to be processed.
    :type text_input: `str`

    :return: Processed text with added black spaces around hyphens.
    :rtype: `str`
    """
    hyphen_pattern = re.compile('(\d)(-)(\w)?')
    return re.sub(hyphen_pattern, r'\1 \2 \3', text_input)


def add_space_around_punctuation(text_input: str) -> str:
    """ Adds a blank space around punctuation.
    
    The characters we consider punctuation are (,.!?/\{}[]()*#@%)

    :param text_input: Text to be processed.
    :type text_input: `str`

    :return: Processed text with added black spaces around hyphens.
    :rtype: `str`
    """
    punctuation_pattern = re.compile('(?<=[^ ])(?=[:,.\\\\!?/\{}\\[\\]()*#@%])|(?<=[:,.\\\\!?/\{}\\[\\]()*#@%])(?=[^ ])')
    
    return re.sub(punctuation_pattern, r' ', text_input)



ABBREVIATIONS = {"hj": "hoje",
                 "vc": "você",
                 "vcs": "vocês",
                 "aq": "aqui",
                 "eh": "é",
                 "cmg": "comigo",
                 "cm": "comigo",
                 "c": "com",
                 "iso": "isso",
                 "mto": "muito",
                 "mt": "muito",
                 "vo": "vou",
                 "p": "para",
                 "pq": "porque",
                 "q": "que",
                 "ñ": "não",
                 "n": "não",
                 "aki": "aqui",
                 "td": "tudo",
                 "blz": "beleza",
                 "ond": "onde",
                 "qndo": "quando",
                 "qnd": "quando",
                 "mnh": "minha",
                 "porq": "porque",
                 "qro": "quero",
                 "agr": "agora",
                 "cod": "código",
                 "codg": "código",
                 "n°": "número",
                 "pagto": "pagamento",
                 "vdd": "verdade",
                 "dps": "depois",
                 "ve": "ver",
                 "pfv": "por favor",
                 "pfvr": "por favor",
                 "av": "avenida",
                 "ap": "apartamento",
                 "tb": "também",
                 "tbm": "também",
                 "cc": "conta corrente",
                 "obg": "obrigado",
                 "obgd": "obrigado",
                 "obgdo": "obrigado",
                 "obgda": "obrigada",
                 "ctt": "contato",
                 "nn": "nao",
                 "qd": "quando",
                 "hr": "hora",
                 "fco": "faço",
                 "qro": "quero",
                 "qlq": "qualquer",
                 "d": "de",
                 "to": "estou",
                 "tô": "estou",
                 "tá": "está",
                 "ta": "está",
                 "msg": "mensagem",
                 "msgs": "mensagens",
                 "msgm": "mensagem",
                 "msgns": "mensagens",
                 "prq": "porque",
                 "vlr": "valor",
                 "qdo": "quando",
                 "apto": "apartamento",
                 "msm": "mesmo",
                 "msmo": "mesmo",
                 "msma": "mesma",
                 "eh": "é",
                 "neh": "né",
                 "itau": "Itaú",
                 "fasso": "faço",
                 "opicao": "opção",
                 "presciso": "preciso",
                 "è": "é",
                 "faser": "fazer",
                 "naõ": "não",
                 "tbem": "também",
                 "extorno": "estorno",
                 "presiso": "preciso",
                 "internete": "internet",
                 "adiquirir": "adquirir",
                 "chipe": "chip",
                 "imail": "email",
                 "e-mail": "email",
                 "tel": "telefone",
                 "poblema": "problema",
                 "whats": "whatsapp",
                 "oque": "o que",
                 "min": "minuto",
                 "tou": "estou",
                 "qto": "quanto",
                 "qtos": "quantos",
                 "qnto": "quanto",
                 "qntos": "quantos",
                 "qta": "quanta",
                 "qtas": "quantas",
                 "qntas": "quantas",
                 "pgto": "pagamento",
                 "pc": "computador",
                 "pq": "porque",
                 "p/": "para",
                 "c/": "com",
                 "br": "brasil",
                 "tv": "televisão",
                 "att": "atenciosamente",
                 "apt": "apartamento",
                 "obs": "observação",
                 "nd": "nada",
                 "aprox": "aproximadamente",
                 "qria": "queria",
                 "qero": "quero",
                 "tmb": "também",
                 "odonto": "odontologia",
                 "gov": "governo",
                 "smp": "sempre",
                 "amg": "amigo",
                 "tds": "todos",
                 "tdos": "todos",
                 "tdas": "todas",
                 "ngm": "ninguém",
                 "bjs": "beijos",
                 "bjos": "beijos",
                 "abç": "abraço",
                 "abçs": "abraços",
                 "num": "número",
                 "mi": "me",
                 "oq": "o que"
                 }



__all__ = ['add_space_to_punctuation',
           'tokenize_text',
           'decision_pipeline',
           'regex_preprocessing',
           'remove_non_ascii_symbol_characters']

from .scanner_tokenization.tokenize_text import tokenize_text
from .add_space_to_puct import add_space_to_punctuation
from .decision_pipeline.pipeline import decision_pipeline
from .regex_preprocessing.regex_callable import regex_preprocessing
from .remove_non_ascii_symbols import remove_non_ascii_symbol_characters



import emoji
import re

from .remove_non_ascii_symbols import remove_non_ascii_symbol_characters


def process_emoji(sentence: str, processing_parameters: list) -> str:
    """Remove non-ascii character but keeps emojis

    The skin tone emoji is removed from the sentence

    :param sentence: Text to be processed.
    :type sentence: `str`

    :param processing_parameters: Preprocessing options to be applied.
    :type processing_parameters: list

    :return: Processed text without non-ascii and with emoji
    :rtype: `str`
    """
    sentence = emoji.demojize(sentence)

    sentence = remove_skin_tone(sentence)

    sentence = remove_non_ascii_symbol_characters(sentence,
                                                  processing_parameters)

    return emoji.emojize(sentence)


def remove_skin_tone(sentence: str) -> str:
    """Remove skin tone emoji  from a sentence

    :param sentence: Text to be processed
    :type sentence: str

    :return: Sentence without skin tone emoji
    :rtype: str
    """
    pattern = r'\s:[A-Za-z]+_skin_tone:'
    return re.sub(pattern, '', sentence)



import typing as tp

from .scanner import Scanner

SCAN = Scanner()


def get_tokens(text_input: str) -> tp.Tuple[tp.List[str]]:
    """ Take Scanner call to tokenize the input text.

    :param text_input: Text to be tokenized.
    :type text_input: `str`
    :return: A tuple containing a list of the tokens and a list of the token types.
    :rtype: `tp.Tuple[tp.List[str]]`
    """
    return SCAN.get_tokens(input_text=text_input)



import typing as tp
import string

import emoji


class Scanner:
    """
    Class for scanning text and finding tokenizing numbers and codes.
    
    Methods:
    -------
        * get_tokens: reads a string and returns a list of tokens and a list of token types.
    """
    def __init__(self):
        """ Initializes the class with character groups. """
        # Attributes
        self.fragment: str  # reference to text fragment received by '__initialize_fragment()'
        self.token: str  # text of token returned by '__next_token()'
        self.token_id: str  # type of token returned
    
        # Protected
        self._letters = set(string.ascii_letters)
        self._letters = self._letters.union(set("ÁÀÂÃáàâãÉÈÊéàêÍÌíìÓÒÔÕóòôõÚÙúùÇç_"))
        self._digit = set("0123456789")
        self._ordinal = set("ºª")
        self._delimiter = set(" \r\n\t\'\"!#%&*()=+{[}]<><;?|@$")
        self._blank = set(" \r\n\t")  # also in self._delimiter
        self._numeric_part = set(":/-.,")  # not in delimiter
    
        self._length: int = 0  # length of text fragment
        self._pos: int = 0  # next character position on the text fragment
        self._current_character = " "  # current character being processed

    def get_tokens(self, input_text: str) -> tp.Tuple[tp.List[str]]:
        """ Reads a string and returns a list of tokens and a list of token types
        
        :param input_text: Text to be scanned.
        :type input_text: `str`
        :return: A tuple containing a list of the tokens and a list of the token types.
        :rtype: `tp.Tuple[tp.List[str]]`
        """
        self.__initialize_fragment(input_text)
        tokens = []
        token_ids = []

        while self.token_id != 'EOT':
            self.__next_token()
            tokens.append(self.token)
            token_ids.append(self.token_id)
        return tokens, token_ids
                        
    def __initialize_fragment(self, fragment: str) -> None:
        """ Initializes the text fragment being scanned.
        
        :param fragment: Text currently being scanned.
        :type fragment: `str`
        """
        self.fragment = fragment
        self.token = ""
        self.token_id = 'INVALID'
        
        self._length = len(self.fragment)
        self._pos = 0
        self._current_character = " "

    def __next_token(self) -> None:
        """ Moves scanner to the next token in the scanned text. """
        self.token = ""
        self.token_id = 'INVALID'
    
        self.__ignore_blanks()
    
        if self._current_character == "\0":
            self.token_id = 'EOT'
            return
    
        if self._current_character in self._letters:
            return self.__get_word()
    
        if self._current_character in self._delimiter.union(self._numeric_part):
            self.token_id = 'DELIMITER'
            self.token = self.token + self._current_character
            self.__get_next_char()
            return
    
        if self._current_character in self._digit:
            return self.__get_number()

        if emoji.is_emoji(self._current_character):
            self.token += self._current_character
            self.__get_next_char()
            return

        self.__get_next_char()
        return

    def __ignore_blanks(self) -> None:
        """ Ignores black characters in scanned text. """
        while self._current_character in self._blank:
            self.__get_next_char()
    
    def __get_next_char(self) -> None:
        """ Moves current character to the next one on the scanned text. """
        if self._pos >= self._length:
            self._current_character = "\0"
            return
        self._current_character = self.fragment[self._pos]
        self._pos += 1
    
    def __get_word(self) -> None:
        """ Matches words on scanned text. """
        self.token_id = 'WORD'
        while self._current_character in self._letters:
            self.token = self.token + self._current_character
            self.__get_next_char()
        if self._current_character in self._digit:
            self.__replace_to_code()

    def __get_number(self) -> None:
        """ Matches numbers on scanned text. """
        self.token_id = 'NUMBER'
        while self._current_character in self._digit.union(self._numeric_part):
            self.token = self.token + self._current_character
            self.__get_next_char()
        if self._current_character in self._letters:
            self.__replace_to_code()

    def __replace_to_code(self) -> None:
        """ Matches codes on scanned text. """
        while self._current_character not in self._blank and self._current_character != '\0':
            self.__set_token('CODE')
            self.__get_next_char()

    def __set_token(self, token_type) -> None:
        """ Set current token and token type. """
        self.token_id = token_type
        self.token = self.token + self._current_character



import typing as tp

from take_text_preprocess.logic.lower_case import to_lower_case
from take_text_preprocess.logic.replace_abbreviations import replace_abbreviations
from take_text_preprocess.logic.abbreviations import ABBREVIATIONS
from take_text_preprocess.logic.add_space_to_puct import add_space_to_punctuation
from take_text_preprocess.logic.remove_non_ascii_symbols import remove_non_ascii_symbol_characters
from take_text_preprocess.logic.regex_preprocessing.regex_callable import regex_preprocessing
from take_text_preprocess.logic.scanner_tokenization.tokenize_text import tokenize_text
from take_text_preprocess.logic.emoji_processing import process_emoji


REGEX_PARAMETERS = ['URL', 'EMAIL']
TOKENIZER_PARAMETERS = ['CODE', 'NUMBER']


def decision_pipeline(line_input: str, processing_parameters=[]) -> str:
    """Apply appropriate preprocessing methods.

    :param line_input: Input text to be processed.
    :type line_input: `str`
    :param processing_parameters: Preprocessing options to be applied.
    :type processing_parameters: tp.List[`str`]
    :return: Preprocessed text.
    :rtype: `str`
    """
    sentence = to_lower_case(line_input)

    if any(parameter in processing_parameters for parameter in REGEX_PARAMETERS):
        sentence = regex_preprocessing(sentence, processing_parameters)

    if any(parameter in processing_parameters for parameter in TOKENIZER_PARAMETERS):
        sentence = tokenize_text(sentence, processing_parameters)

    if 'EMOJI' in processing_parameters:
       sentence = process_emoji(sentence, processing_parameters)
    else:
        sentence = remove_non_ascii_symbol_characters(sentence,
                                                      processing_parameters)

    sentence = add_space_to_punctuation(sentence)

    if 'ABBR' in processing_parameters:
        sentence = replace_abbreviations(sentence, ABBREVIATIONS)

    return sentence



import typing as tp

from .replace_regex import replace_url, replace_email


def regex_preprocessing(text_input: str, processing_parameters: tp.List[str]) -> str:
    """Apply regex preprocessing methods on text.

    :param text_input: Input to be processed.
    :type text_input: `str`
    :param processing_parameters: Processing types to be applied. Current types supported are 'url' and 'email'.
    :type processing_parameters: `tp.List[str]`
    :return: Processed text
    :rtype: `str`
    """
    sentence = text_input
    if 'URL' in processing_parameters:
        sentence = replace_url(sentence)
    if 'EMAIL' in processing_parameters:
        sentence = replace_email(sentence)
    return sentence



import re


def replace_url(text_input: str) -> str:
    """Replace URL with <URL> token on input text.
    :param text_input: Input text to be processed.
    :type text_input: `str`
    :return: Text with URL replaced by <URL>.
    :rtype: `str`
    """
    url_regex = re.compile(r'((https?:\/\/)|(www\.))\w+\.\w+(\.\w+)*(\/.+)*')
    processed_sentence, _ = url_regex.subn('URL', text_input)
    return processed_sentence


def replace_email(text_input: str) -> str:
    """Replace e-mails with <EMAIL> token on input text
    :param text_input: Input text to be processed.
    :type text_input: `str`
    :return: Text with Email replaced by <EMAIL>.
    :rtype: `str`
    """
    if text_input.find('@'):
        email_regex = re.compile(r'([a-z.0-9\_]+)@\w+\.\w+(\.br)?')
        processed_sentence, _ = email_regex.subn('EMAIL', text_input)
        return processed_sentence
    else:
        return text_input



import string
import re


def set_valid_characters(processing_parameter: list) -> set:
    """Set valid characters to keep

    :param processing_parameter: Processing types to be applied.
    :type processing_parameter: `list`
    :return: set with characters to keep
    :rtype: `set`
    """
    accented_letters = {"á", "é", "í", "ó", "ú", "à", "â", "ê", "ô", "ã", "õ",
                        "ç"}
    if "SYMBOL" in processing_parameter:
        valid_characters = set(string.ascii_letters)
        punctuation_to_keep = {"!", ",", ".", ";", "?", ":", "_", "-", "’"}
        valid_characters = valid_characters.union(punctuation_to_keep)
        valid_characters.add(" ")
    else:
        valid_characters = set(string.printable)
    valid_characters = valid_characters.union(accented_letters)
    return valid_characters


def remove_non_ascii_symbol_characters(text_input: str,
                                       processing_parameter: list = []) -> str:
    """ Remove non-ascii characters and symbols from input text.

    To remove symbols value "SYMBOL" has to be set on processing_parameter.

    :param text_input: Input text to be processed.
    :type text_input: `str`
    :param processing_parameter: Processing types to be applied.
    :type processing_parameter: `list`

    :return: Processed text
    :rtype: `str`
    """
    valid_characters = set_valid_characters(processing_parameter)

    return re.sub(" +", " ",
                  "".join(filter(lambda x: x in valid_characters, text_input)))



from setuptools import setup, find_packages

author = 'Data & Analytics Research'
author_email = 'analytics.dar@take.net'
version = '0.0.6b3'
license = 'MIT License'
credits = ['Milo Utsch', 'Gabriel Oliveira']
name = 'take-text-preprocess'
maintainer = "daresearch"
maintainer_email = "anaytics.dar@take.net"
keywords = ['text', 'preprocessing']
description = 'Text Preprocesser'
long_description = open('README.md').read()
long_description_content_type = 'text/markdown'
classifiers = [
    'Programming Language :: Python :: 3',
    'Operating System :: OS Independent'
]

install_requires = [
    req
    for req in [
        line.split("#", 1)[0].strip()
        for line in open("requirements.txt", "r", encoding="utf-8")
    ]
    if req and not req.startswith("--")
]

setup(
    name=name,
    author=author,
    author_email=author_email,
    version=version,
    license=license,
    credits=credits,
    maintainer=maintainer,
    maintainer_email=maintainer_email,
    keywords=keywords,
    description=description,
    long_description=long_description,
    long_description_content_type=long_description_content_type,
    classifiers=classifiers,
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    install_requires=install_requires
)



import typing as tp


def replace_abbreviations(sentence: str,
                          abbreviations: tp.Dict[str, str]) -> str:
    """
    Checks if there is abbreviated words on the sentence and replaces it with
    the full word.

    :param sentence: Sentence to be processed.
    :type sentence: `str`
    :param abbreviations: Dictionary containing word abbreviations and their full form.
    :type abbreviations: `tp.Dict[str, str]`
    :return: Sentence with all words in their full form.
    :rtype: `str`
    """
    sentence_split = sentence.split()
    modified_sentence = []
    for word in sentence_split:
        if word in abbreviations:
            modified_sentence += abbreviations[word].split()
        else:
            modified_sentence += [word]
    return ' '.join(modified_sentence)



import typing as tp

from .scanner_get_tokens import get_tokens


def tokenize_text(text_input: str, processing_parameters: tp.List[str] = []) -> str:
    """ Replace Numbers and Codes with tokens.
    
    :param text_input: Input to be tokenized.
    :type text_input: `str`
    :param processing_parameters: Processing types to be applied. Current types supported are 'NUMBER' or 'CODE'.
    :type processing_parameters: `tp.List[str]`
    :return: Tokenized text.
    :rtype: `str`
    """
    tokens, tokens_type = get_tokens(text_input)
    tokenized_sentence = []
    
    for token, token_type in zip(tokens, tokens_type):
        if token_type in processing_parameters:
            tokenized_sentence.append(token_type)
        else:
            tokenized_sentence.append(token)
        
    return ' '.join(tokenized_sentence)[:-1]



import typing as tp

from take_text_preprocess.data.validate_inputs import validate_type
from take_text_preprocess.logic.decision_pipeline.pipeline import decision_pipeline


def pre_process(text_input: str, optional_tokenization: tp.List[str] = []):
    """ Apply appropriate pre processing methods to input text.
    
    Optional tokenization includes: `DOC`, `CEP`, `PHONE`, `NUMBER`, `CODE`, `URL`, `CEP`.

    :param text_input: Input text to be processed.
    :type text_input: `str`
    :param optional_tokenization: Optional pre processing options to be applied. Defaults to basic pre processing.
    :type optional_tokenization: tp.List[`str`]
    :return: Pre processed text.
    :rtype: `str`
    """
    validate_type(text_input)
    return decision_pipeline(text_input, optional_tokenization)
